@@ -1,22 +1,11 @@
 // Copyright (C) 2019-2020 François Laignel <fengalin@free.fr>
 // Copyright (C) 2020 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
-//
-// SPDX-License-Identifier: LGPL-2.1-or-later
+// SPDX-License-Identifier: MPL-2.0
 #![allow(clippy::non_send_fields_in_send_ty, unused_doc_comments)]
 
 use futures::channel::mpsc;
@@ -24,14 +13,14 @@ use futures::prelude::*;
 
 use gst::glib;
 
+use gst::EventView;
 use gst::prelude::*;
 use gst::subclass::prelude::*;
-use gst::EventView;
 
 use std::sync::LazyLock;
 
-use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Mutex;
+use std::sync::atomic::{AtomicBool, Ordering};
 use std::time::Duration;
 
 use gstthreadshare::runtime::prelude::*;
@@ -91,10 +80,12 @@ mod imp_src {
 
             let ret = match event.view() {
                 EventView::FlushStart(..) => {
-                    imp.task.flush_start().await_maybe_on_context().is_ok()
+                    imp.task.flush_start().block_on_or_add_subtask(pad).is_ok()
                 }
                 EventView::Qos(..) | EventView::Reconfigure(..) | EventView::Latency(..) => true,
-                EventView::FlushStop(..) => imp.task.flush_stop().await_maybe_on_context().is_ok(),
+                EventView::FlushStop(..) => {
+                    imp.task.flush_stop().block_on_or_add_subtask(pad).is_ok()
+                }
                 _ => false,
             };
 
@@ -123,7 +114,7 @@ mod imp_src {
     impl ElementSrcTestTask {
         fn flush(&mut self) {
             // Purge the channel
-            while let Ok(Some(_item)) = self.receiver.try_next() {}
+            while let Ok(_item) = self.receiver.try_recv() {}
         }
         async fn push_item(&self, item: Item) -> Result<gst::FlowSuccess, gst::FlowError> {
             gst::debug!(SRC_CAT, obj = self.element, "Handling {:?}", item);
@@ -144,6 +135,10 @@ mod imp_src {
     impl TaskImpl for ElementSrcTestTask {
         type Item = Item;
 
+        fn obj(&self) -> &impl IsA<glib::Object> {
+            &self.element
+        }
+
         async fn try_next(&mut self) -> Result<Item, gst::FlowError> {
             self.receiver.next().await.ok_or_else(|| {
                 gst::log!(SRC_CAT, obj = self.element, "SrcPad channel aborted");
@@ -219,44 +214,52 @@ mod imp_src {
 
             let (sender, receiver) = mpsc::channel(1);
             *self.sender.lock().unwrap() = Some(sender);
-
             self.task
                 .prepare(
                     ElementSrcTestTask::new(self.obj().clone(), receiver),
                     context,
                 )
-                .block_on()?;
-
-            gst::debug!(SRC_CAT, imp = self, "Prepared");
-
-            Ok(())
+                .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                    if res.is_ok() {
+                        gst::debug!(SRC_CAT, obj = elem, "Prepared");
+                    }
+                })
         }
 
         fn unprepare(&self) {
             gst::debug!(SRC_CAT, imp = self, "Unpreparing");
+            let _ = self
+                .task
+                .unprepare()
+                .block_on_or_add_subtask_then(self.obj(), |elem, _| {
+                    *elem.imp().sender.lock().unwrap() = None;
 
-            *self.sender.lock().unwrap() = None;
-            self.task.unprepare().block_on().unwrap();
-
-            gst::debug!(SRC_CAT, imp = self, "Unprepared");
+                    gst::debug!(SRC_CAT, obj = elem, "Unprepared");
+                });
         }
 
         fn stop(&self) {
             gst::debug!(SRC_CAT, imp = self, "Stopping");
-            self.task.stop().await_maybe_on_context().unwrap();
-            gst::debug!(SRC_CAT, imp = self, "Stopped");
+            self.task
+                .stop()
+                .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                    if res.is_ok() {
+                        gst::debug!(SRC_CAT, obj = elem, "Stopped");
+                    }
+                })
+                .unwrap();
         }
 
         fn start(&self) {
             gst::debug!(SRC_CAT, imp = self, "Starting");
-            self.task.start().await_maybe_on_context().unwrap();
-            gst::debug!(SRC_CAT, imp = self, "Started");
-        }
-
-        fn pause(&self) {
-            gst::debug!(SRC_CAT, imp = self, "Pausing");
-            self.task.pause().block_on().unwrap();
-            gst::debug!(SRC_CAT, imp = self, "Paused");
+            self.task
+                .start()
+                .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                    if res.is_ok() {
+                        gst::debug!(SRC_CAT, obj = elem, "Started");
+                    }
+                })
+                .unwrap();
         }
     }
 
@@ -282,12 +285,14 @@ mod imp_src {
     impl ObjectImpl for ElementSrcTest {
         fn properties() -> &'static [glib::ParamSpec] {
             static PROPERTIES: LazyLock<Vec<glib::ParamSpec>> = LazyLock::new(|| {
-                vec![glib::ParamSpecString::builder("context")
-                    .nick("Context")
-                    .blurb("Context name to share threads with")
-                    .default_value(Some(DEFAULT_CONTEXT))
-                    .write_only()
-                    .build()]
+                vec![
+                    glib::ParamSpecString::builder("context")
+                        .nick("Context")
+                        .blurb("Context name to share threads with")
+                        .default_value(Some(DEFAULT_CONTEXT))
+                        .write_only()
+                        .build(),
+                ]
             });
 
             PROPERTIES.as_ref()
@@ -363,8 +368,8 @@ mod imp_src {
                         gst::StateChangeError
                     })?;
                 }
-                gst::StateChange::PlayingToPaused => {
-                    self.pause();
+                gst::StateChange::PausedToReady => {
+                    self.stop();
                 }
                 gst::StateChange::ReadyToNull => {
                     self.unprepare();
@@ -375,9 +380,6 @@ mod imp_src {
             let mut success = self.parent_change_state(transition)?;
 
             match transition {
-                gst::StateChange::PausedToReady => {
-                    self.stop();
-                }
                 gst::StateChange::PausedToPlaying => {
                     self.start();
                 }
@@ -393,10 +395,16 @@ mod imp_src {
         fn send_event(&self, event: gst::Event) -> bool {
             match event.view() {
                 EventView::FlushStart(..) => {
-                    self.task.flush_start().await_maybe_on_context().unwrap();
+                    self.task
+                        .flush_start()
+                        .block_on_or_add_subtask(&*self.obj())
+                        .unwrap();
                 }
                 EventView::FlushStop(..) => {
-                    self.task.flush_stop().await_maybe_on_context().unwrap();
+                    self.task
+                        .flush_stop()
+                        .block_on_or_add_subtask(&*self.obj())
+                        .unwrap();
                 }
                 _ => (),
             }
@@ -566,12 +574,14 @@ mod imp_sink {
     impl ObjectImpl for ElementSinkTest {
         fn properties() -> &'static [glib::ParamSpec] {
             static PROPERTIES: LazyLock<Vec<glib::ParamSpec>> = LazyLock::new(|| {
-                vec![glib::ParamSpecBoxed::builder::<ItemSender>("sender")
-                    .nick("Sender")
-                    .blurb("Channel sender to forward the incoming items to")
-                    .write_only()
-                    .construct_only()
-                    .build()]
+                vec![
+                    glib::ParamSpecBoxed::builder::<ItemSender>("sender")
+                        .nick("Sender")
+                        .blurb("Channel sender to forward the incoming items to")
+                        .write_only()
+                        .construct_only()
+                        .build(),
+                ]
             });
 
             PROPERTIES.as_ref()
@@ -616,6 +626,11 @@ mod imp_sink {
             Some(&*ELEMENT_METADATA)
         }
 
+        fn send_event(&self, event: gst::Event) -> bool {
+            gst::log!(SINK_CAT, imp = self, "Got {event:?}");
+            self.sink_pad.gst_pad().push_event(event)
+        }
+
         fn pad_templates() -> &'static [gst::PadTemplate] {
             static PAD_TEMPLATES: LazyLock<Vec<gst::PadTemplate>> = LazyLock::new(|| {
                 let caps = gst::Caps::new_any();
@@ -778,7 +793,7 @@ fn nominal_scenario(
         .unwrap();
 
     // Nothing forwarded
-    receiver.try_next().unwrap_err();
+    receiver.try_recv().unwrap_err();
 
     // Switch back the Pad task to Started
     pipeline.set_state(gst::State::Playing).unwrap();
@@ -998,7 +1013,7 @@ fn start_pause_start() {
     }
 
     // Nothing else forwarded
-    receiver.try_next().unwrap_err();
+    receiver.try_recv().unwrap_err();
 
     pipeline.set_state(gst::State::Null).unwrap();
 }
