@@ -1,23 +1,22 @@
 // SPDX-License-Identifier: MPL-2.0
 
-use crate::signaller::{prelude::*, Signallable};
-use crate::utils::{gvalue_to_json, serialize_json_object};
 use crate::RUNTIME;
-use anyhow::{anyhow, Error};
+use crate::signaller::{Signallable, prelude::*};
+use crate::utils::{create_tls_connector, gvalue_to_json, serialize_json_object};
+use anyhow::{Error, anyhow};
+use async_tungstenite::tungstenite::Message as WsMessage;
 use async_tungstenite::tungstenite::client::IntoClientRequest;
 use async_tungstenite::tungstenite::http::{HeaderName, HeaderValue};
-use async_tungstenite::tungstenite::Message as WsMessage;
 use futures::channel::mpsc;
 use futures::prelude::*;
 use gst::glib;
 use gst::glib::prelude::*;
 use gst::subclass::prelude::*;
 use gst_plugin_webrtc_protocol as p;
-use once_cell::sync::Lazy;
 use std::collections::{HashMap, HashSet};
 use std::ops::ControlFlow;
 use std::str::FromStr;
-use std::sync::Mutex;
+use std::sync::{LazyLock, Mutex};
 use std::time::Duration;
 use tokio::{task, time::timeout};
 use url::Url;
@@ -43,6 +42,7 @@ pub struct Settings {
     role: WebRTCSignallerRole,
     headers: Option<gst::Structure>,
     insecure_tls: bool,
+    connect_to_first_producer: bool,
 }
 
 impl Default for Settings {
@@ -54,6 +54,7 @@ impl Default for Settings {
             role: Default::default(),
             headers: None,
             insecure_tls: DEFAULT_INSECURE_TLS,
+            connect_to_first_producer: false,
         }
     }
 }
@@ -68,6 +69,7 @@ pub struct Signaller {
 struct State {
     /// Sender for the websocket messages
     websocket_sender: Option<mpsc::Sender<p::IncomingMessage>>,
+    connect_task_handle: Option<task::JoinHandle<()>>,
     send_task_handle: Option<task::JoinHandle<Result<(), Error>>>,
     receive_task_handle: Option<task::JoinHandle<()>>,
     producers: HashSet<String>,
@@ -79,6 +81,14 @@ impl Signaller {
         self.settings.lock().unwrap().uri.clone()
     }
 
+    pub fn connect_to_first_producer(&self) -> bool {
+        self.settings.lock().unwrap().connect_to_first_producer
+    }
+
+    pub fn set_connect_to_first_producer(&self, value: bool) {
+        self.settings.lock().unwrap().connect_to_first_producer = value;
+    }
+
     fn set_uri(&self, uri: &str) -> Result<(), Error> {
         let mut settings = self.settings.lock().unwrap();
         let mut uri = Url::from_str(uri).map_err(|err| anyhow!("{err:?}"))?;
@@ -99,53 +109,72 @@ impl Signaller {
             }
         }
 
+        if let Some(connect_to_first_producer) = uri
+            .query_pairs()
+            .find(|(k, _)| k == "connect-to-first-producer")
+            .map(|v| matches!(v.1.to_lowercase().as_str(), "true" | "1" | ""))
+        {
+            if !matches!(settings.role, WebRTCSignallerRole::Consumer) {
+                gst::warning!(
+                    CAT,
+                    "Setting connect-to-first-producer doesn't make sense for {:?}",
+                    settings.role
+                );
+            } else {
+                settings.connect_to_first_producer = connect_to_first_producer;
+            }
+        }
+
         if let Some(peer_id) = &settings.producer_peer_id {
             uri.query_pairs_mut()
                 .clear()
                 .append_pair("peer-id", peer_id);
         }
 
+        if settings.connect_to_first_producer {
+            uri.query_pairs_mut()
+                .clear()
+                .append_pair("connect-to-first-producer", "true");
+        }
+
         settings.uri = uri;
 
         Ok(())
     }
 
     async fn connect(&self) -> Result<(), Error> {
-        let (cafile, insecure_tls, role) = {
+        let (cafile, insecure_tls, role, connect_to_first_producer) = {
             let settings = self.settings.lock().unwrap();
             (
                 settings.cafile.clone(),
                 settings.insecure_tls,
                 settings.role,
+                settings.connect_to_first_producer,
             )
         };
 
-        if let super::WebRTCSignallerRole::Consumer = role {
-            self.producer_peer_id()
-                .ok_or_else(|| anyhow!("No target producer peer id set"))?;
-        }
-
-        let mut connector_builder = tokio_native_tls::native_tls::TlsConnector::builder();
-
-        if let Some(path) = cafile {
-            let cert = tokio::fs::read_to_string(&path).await?;
-            let cert = tokio_native_tls::native_tls::Certificate::from_pem(cert.as_bytes())?;
-            connector_builder.add_root_certificate(cert);
+        if let super::WebRTCSignallerRole::Consumer = role
+            && !connect_to_first_producer
+            && self.producer_peer_id().is_none()
+        {
+            gst::info!(
+                CAT,
+                imp = self,
+                "No producer peer id set, listening for producer session requests"
+            );
         }
 
         if insecure_tls {
-            connector_builder.danger_accept_invalid_certs(true);
-            gst::warning!(CAT, imp: self, "insecure tls connections are allowed");
+            gst::warning!(CAT, imp = self, "insecure tls connections are allowed");
         }
 
-        let connector = Some(tokio_native_tls::TlsConnector::from(
-            connector_builder.build()?,
-        ));
-
+        let connector = create_tls_connector(cafile, insecure_tls)
+            .map_ok(Some)
+            .await?;
         let mut uri = self.uri();
         uri.set_query(None);
 
-        gst::info!(CAT, imp: self, "connecting to {}", uri.to_string());
+        gst::info!(CAT, imp = self, "connecting to {}", uri.to_string());
 
         let mut req = uri.into_client_request()?;
         let req_headers = req.headers_mut();
@@ -165,7 +194,7 @@ impl Signaller {
         )
         .await??;
 
-        gst::info!(CAT, imp: self, "connected");
+        gst::info!(CAT, imp = self, "connected");
 
         // Channel for asynchronously sending out websocket message
         let (mut ws_sink, mut ws_stream) = ws.split();
@@ -173,57 +202,51 @@ impl Signaller {
         // 1000 is completely arbitrary, we simply don't want infinite piling
         // up of messages as with unbounded
         let (websocket_sender, mut websocket_receiver) = mpsc::channel::<p::IncomingMessage>(1000);
-        let send_task_handle =
-            RUNTIME.spawn(glib::clone!(@weak-allow-none self as this => async move {
+        let send_task_handle = RUNTIME.spawn(glib::clone!(
+            #[to_owned(rename_to = this)]
+            self,
+            async move {
                 let mut res = Ok(());
                 while let Some(msg) = websocket_receiver.next().await {
                     gst::log!(CAT, "Sending websocket message {:?}", msg);
                     res = ws_sink
-                        .send(WsMessage::Text(serde_json::to_string(&msg).unwrap()))
+                        .send(WsMessage::text(serde_json::to_string(&msg).unwrap()))
                         .await;
 
                     if let Err(ref err) = res {
-                        this.as_ref().map_or_else(|| gst::error!(CAT, "Quitting send loop: {err}"),
-                            |this| gst::error!(CAT, imp: this, "Quitting send loop: {err}")
-                        );
+                        gst::error!(CAT, imp = this, "Quitting send loop: {err}");
                         break;
                     }
                 }
 
-                this.map_or_else(|| gst::debug!(CAT, "Done sending"),
-                    |this| gst::debug!(CAT, imp: this, "Done sending")
-                );
+                gst::debug!(CAT, imp = this, "Done sending");
 
-                let _ = ws_sink.close().await;
+                let _ = ws_sink.close(None).await;
 
                 res.map_err(Into::into)
-            }));
+            }
+        ));
 
         let obj = self.obj();
-        let meta =
-            if let Some(meta) = obj.emit_by_name::<Option<gst::Structure>>("request-meta", &[]) {
-                gvalue_to_json(&meta.to_value())
-            } else {
-                None
-            };
+        let meta = match obj.emit_by_name::<Option<gst::Structure>>("request-meta", &[]) {
+            Some(meta) => gvalue_to_json(&meta.to_value()),
+            _ => None,
+        };
 
-        let receive_task_handle =
-            RUNTIME.spawn(glib::clone!(@weak-allow-none self as this => async move {
+        let receive_task_handle = RUNTIME.spawn(glib::clone!(
+            #[to_owned(rename_to = this)]
+            self,
+            async move {
                 while let Some(msg) = tokio_stream::StreamExt::next(&mut ws_stream).await {
-                    if let Some(ref this) = this {
-                        if let ControlFlow::Break(_) = this.handle_message(msg, &meta) {
-                            break;
-                        }
-                    } else {
+                    if let ControlFlow::Break(_) = this.handle_message(msg, &meta) {
                         break;
                     }
                 }
 
                 let msg = "Stopped websocket receiving";
-                this.map_or_else(|| gst::info!(CAT, "{msg}"),
-                    |this| gst::info!(CAT, imp: this, "{msg}")
-                );
-            }));
+                gst::info!(CAT, imp = this, "{msg}");
+            }
+        ));
 
         let mut state = self.state.lock().unwrap();
         state.websocket_sender = Some(websocket_sender);
@@ -235,13 +258,18 @@ impl Signaller {
 
     fn set_status(&self, meta: &Option<serde_json::Value>, peer_id: &str) {
         self.state.lock().unwrap().client_id = Some(peer_id.to_string());
+        self.obj().notify("client-id");
 
         let role = self.settings.lock().unwrap().role;
         self.send(p::IncomingMessage::SetPeerStatus(match role {
             super::WebRTCSignallerRole::Consumer => p::PeerStatus {
                 meta: meta.clone(),
                 peer_id: Some(peer_id.to_string()),
-                roles: vec![],
+                roles: if self.connect_to_first_producer() || self.producer_peer_id().is_some() {
+                    vec![]
+                } else {
+                    vec![p::PeerRole::Consumer]
+                },
             },
             super::WebRTCSignallerRole::Producer => p::PeerStatus {
                 meta: meta.clone(),
@@ -277,12 +305,12 @@ impl Signaller {
 
                 for (key, value) in structure.iter() {
                     if let Ok(Ok(value_str)) = value.transform::<String>().map(|v| v.get()) {
-                        gst::log!(CAT, imp: self, "headers '{}' -> '{}'", key, value_str);
+                        gst::log!(CAT, imp = self, "headers '{}' -> '{}'", key, value_str);
                         hash.insert(key.to_string(), value_str);
                     } else {
                         gst::warning!(
                             CAT,
-                            imp: self,
+                            imp = self,
                             "Failed to convert headers '{}' to string ('{:?}')",
                             key,
                             value
@@ -297,11 +325,16 @@ impl Signaller {
     fn send(&self, msg: p::IncomingMessage) {
         let state = self.state.lock().unwrap();
         if let Some(mut sender) = state.websocket_sender.clone() {
-            RUNTIME.spawn(glib::clone!(@weak self as this => async move {
-                if let Err(err) = sender.send(msg).await {
-                    this.obj().emit_by_name::<()>("error", &[&format!("Error: {}", err)]);
+            RUNTIME.spawn(glib::clone!(
+                #[to_owned(rename_to = this)]
+                self,
+                async move {
+                    if let Err(err) = sender.send(msg).await {
+                        this.obj()
+                            .emit_by_name::<()>("error", &[&format!("Error: {err}")]);
+                    }
                 }
-            }));
+            ));
         }
     }
 
@@ -312,11 +345,12 @@ impl Signaller {
 
             self.send(p::IncomingMessage::StartSession(p::StartSessionMessage {
                 peer_id: target_producer.clone(),
+                offer: None,
             }));
 
             gst::info!(
                 CAT,
-                imp: self,
+                imp = self,
                 "Started session with producer peer id {target_producer}",
             );
         }
@@ -329,13 +363,17 @@ impl Signaller {
     ) -> ControlFlow<()> {
         match msg {
             Ok(WsMessage::Text(msg)) => {
-                gst::trace!(CAT, imp: self, "Received message {}", msg);
+                gst::trace!(CAT, imp = self, "Received message {}", msg);
 
                 if let Ok(msg) = serde_json::from_str::<p::OutgoingMessage>(&msg) {
                     match msg {
                         p::OutgoingMessage::Welcome { peer_id } => {
                             self.set_status(meta, &peer_id);
-                            self.start_session();
+                            if self.producer_peer_id().is_some() {
+                                self.start_session();
+                            } else if self.connect_to_first_producer() {
+                                self.send(p::IncomingMessage::List);
+                            }
                         }
                         p::OutgoingMessage::PeerStatusChanged(p::PeerStatus {
                             meta,
@@ -345,7 +383,7 @@ impl Signaller {
                             let meta = meta.and_then(|m| match m {
                                 serde_json::Value::Object(v) => Some(serialize_json_object(&v)),
                                 _ => {
-                                    gst::error!(CAT, imp: self, "Invalid json value: {m:?}");
+                                    gst::error!(CAT, imp = self, "Invalid json value: {m:?}");
                                     None
                                 }
                             });
@@ -380,23 +418,47 @@ impl Signaller {
                         p::OutgoingMessage::StartSession {
                             session_id,
                             peer_id,
+                            offer,
                         } => {
                             assert!(matches!(
                                 self.obj().property::<WebRTCSignallerRole>("role"),
                                 super::WebRTCSignallerRole::Producer
                             ));
 
+                            let sdp = {
+                                if let Some(offer) = offer {
+                                    match gst_sdp::SDPMessage::parse_buffer(offer.as_bytes()) {
+                                        Ok(sdp) => Some(sdp),
+                                        Err(err) => {
+                                            self.obj().emit_by_name::<()>(
+                                                "error",
+                                                &[&format!("Error parsing SDP: {offer} {err:?}")],
+                                            );
+
+                                            return ControlFlow::Break(());
+                                        }
+                                    }
+                                } else {
+                                    None
+                                }
+                            };
+
                             self.obj().emit_by_name::<()>(
                                 "session-requested",
                                 &[
                                     &session_id,
                                     &peer_id,
-                                    &None::<gst_webrtc::WebRTCSessionDescription>,
+                                    &sdp.map(|sdp| {
+                                        gst_webrtc::WebRTCSessionDescription::new(
+                                            gst_webrtc::WebRTCSDPType::Offer,
+                                            sdp,
+                                        )
+                                    }),
                                 ],
                             );
                         }
                         p::OutgoingMessage::EndSession(p::EndSessionMessage { session_id }) => {
-                            gst::info!(CAT, imp: self, "Session {session_id} ended");
+                            gst::info!(CAT, imp = self, "Session {session_id} ended");
 
                             self.obj()
                                 .emit_by_name::<bool>("session-ended", &[&session_id]);
@@ -445,6 +507,21 @@ impl Signaller {
                             }
                         },
                         p::OutgoingMessage::List { producers } => {
+                            let mut settings = self.settings.lock().unwrap();
+                            let role = settings.role;
+                            if matches!(role, super::WebRTCSignallerRole::Consumer)
+                                && settings.connect_to_first_producer
+                                && settings.producer_peer_id.is_none()
+                                && let Some(producer) = producers.first()
+                            {
+                                settings.producer_peer_id = Some(producer.id.clone());
+
+                                drop(settings);
+
+                                self.start_session();
+
+                                return ControlFlow::Continue(());
+                            }
                             for producer in producers {
                                 let mut state = self.state.lock().unwrap();
                                 if !state.producers.contains(&producer.id) {
@@ -452,9 +529,15 @@ impl Signaller {
                                     drop(state);
 
                                     let meta = producer.meta.and_then(|m| match m {
-                                        serde_json::Value::Object(v) => Some(serialize_json_object(&v)),
+                                        serde_json::Value::Object(v) => {
+                                            Some(serialize_json_object(&v))
+                                        }
                                         _ => {
-                                            gst::error!(CAT, imp: self, "Invalid json value: {m:?}");
+                                            gst::error!(
+                                                CAT,
+                                                imp = self,
+                                                "Invalid json value: {m:?}"
+                                            );
                                             None
                                         }
                                     });
@@ -466,6 +549,7 @@ impl Signaller {
                                 }
                             }
                         }
+                        p::OutgoingMessage::ListConsumers { .. } => todo!(),
                         p::OutgoingMessage::Error { details } => {
                             self.obj().emit_by_name::<()>(
                                 "error",
@@ -474,22 +558,22 @@ impl Signaller {
                         }
                     }
                 } else {
-                    gst::error!(CAT, imp: self, "Unknown message from server: {}", msg);
+                    gst::error!(CAT, imp = self, "Unknown message from server: {}", msg);
 
                     self.obj().emit_by_name::<()>(
                         "error",
-                        &[&format!("Unknown message from server: {}", msg)],
+                        &[&format!("Unknown message from server: {msg}")],
                     );
                 }
             }
             Ok(WsMessage::Close(reason)) => {
-                gst::info!(CAT, imp: self, "websocket connection closed: {:?}", reason);
+                gst::info!(CAT, imp = self, "websocket connection closed: {:?}", reason);
                 return ControlFlow::Break(());
             }
             Ok(_) => (),
             Err(err) => {
                 self.obj()
-                    .emit_by_name::<()>("error", &[&format!("Error receiving: {}", err)]);
+                    .emit_by_name::<()>("error", &[&format!("Error receiving: {err}")]);
                 return ControlFlow::Break(());
             }
         }
@@ -507,8 +591,14 @@ impl ObjectSubclass for Signaller {
 
 impl ObjectImpl for Signaller {
     fn properties() -> &'static [glib::ParamSpec] {
-        static PROPS: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
+        static PROPS: LazyLock<Vec<glib::ParamSpec>> = LazyLock::new(|| {
             vec![
+                glib::ParamSpecBoolean::builder("manual-sdp-munging")
+                    .nick("Manual SDP munging")
+                    .blurb("Whether the signaller manages SDP munging itself")
+                    .default_value(false)
+                    .read_only()
+                    .build(),
                 glib::ParamSpecString::builder("uri")
                     .nick("Signaller URI")
                     .blurb("URI for connecting to the signaller server")
@@ -604,6 +694,7 @@ impl ObjectImpl for Signaller {
     fn property(&self, _id: usize, pspec: &glib::ParamSpec) -> glib::Value {
         let settings = self.settings.lock().unwrap();
         match pspec.name() {
+            "manual-sdp-munging" => false.to_value(),
             "uri" => settings.uri.to_string().to_value(),
             "producer-peer-id" => {
                 if !matches!(settings.role, WebRTCSignallerRole::Consumer) {
@@ -628,32 +719,53 @@ impl ObjectImpl for Signaller {
 
 impl SignallableImpl for Signaller {
     fn start(&self) {
-        gst::info!(CAT, imp: self, "Starting");
-        RUNTIME.spawn(glib::clone!(@weak self as this => async move {
-            if let Err(err) = this.connect().await {
-                this.obj().emit_by_name::<()>("error", &[&format!("Error receiving: {}", err)]);
+        gst::info!(CAT, imp = self, "Starting");
+
+        let mut state = self.state.lock().unwrap();
+        let connect_task_handle = RUNTIME.spawn(glib::clone!(
+            #[to_owned(rename_to = this)]
+            self,
+            async move {
+                if let Err(err) = this.connect().await {
+                    this.obj()
+                        .emit_by_name::<()>("error", &[&format!("Error receiving: {err}")]);
+                }
             }
-        }));
+        ));
+
+        state.connect_task_handle = Some(connect_task_handle);
     }
 
     fn stop(&self) {
-        gst::info!(CAT, imp: self, "Stopping now");
+        gst::info!(CAT, imp = self, "Stopping now");
 
         let mut state = self.state.lock().unwrap();
+
+        // First make sure the connect task is stopped if it is still
+        // running
+        let connect_task_handle = state.connect_task_handle.take();
+        if let Some(handle) = connect_task_handle {
+            RUNTIME.block_on(async move {
+                handle.abort();
+                let _ = handle.await;
+            });
+        }
+
         let send_task_handle = state.send_task_handle.take();
         let receive_task_handle = state.receive_task_handle.take();
         if let Some(mut sender) = state.websocket_sender.take() {
             RUNTIME.block_on(async move {
                 sender.close_channel();
 
-                if let Some(handle) = send_task_handle {
-                    if let Err(err) = handle.await {
-                        gst::warning!(CAT, imp: self, "Error while joining send task: {}", err);
-                    }
+                if let Some(handle) = send_task_handle
+                    && let Err(err) = handle.await
+                {
+                    gst::warning!(CAT, imp = self, "Error while joining send task: {}", err);
                 }
 
                 if let Some(handle) = receive_task_handle {
                     handle.abort();
+                    let _ = handle.await;
                 }
             });
         }
@@ -662,22 +774,21 @@ impl SignallableImpl for Signaller {
     }
 
     fn send_sdp(&self, session_id: &str, sdp: &gst_webrtc::WebRTCSessionDescription) {
-        gst::debug!(CAT, imp: self, "Sending SDP {sdp:#?}");
-
-        let role = self.settings.lock().unwrap().role;
-        let is_consumer = matches!(role, super::WebRTCSignallerRole::Consumer);
+        gst::debug!(CAT, imp = self, "Sending SDP {sdp:#?}");
 
         let msg = p::IncomingMessage::Peer(p::PeerMessage {
             session_id: session_id.to_owned(),
-            peer_message: p::PeerMessageInner::Sdp(if is_consumer {
-                p::SdpMessage::Answer {
-                    sdp: sdp.sdp().as_text().unwrap(),
-                }
-            } else {
-                p::SdpMessage::Offer {
-                    sdp: sdp.sdp().as_text().unwrap(),
-                }
-            }),
+            peer_message: p::PeerMessageInner::Sdp(
+                if sdp.type_() == gst_webrtc::WebRTCSDPType::Offer {
+                    p::SdpMessage::Offer {
+                        sdp: sdp.sdp().as_text().unwrap(),
+                    }
+                } else {
+                    p::SdpMessage::Answer {
+                        sdp: sdp.sdp().as_text().unwrap(),
+                    }
+                },
+            ),
         });
 
         self.send(msg);
@@ -692,7 +803,7 @@ impl SignallableImpl for Signaller {
     ) {
         gst::debug!(
             CAT,
-            imp: self,
+            imp = self,
             "Adding ice candidate {candidate:?} for {sdp_m_line_index:?} on session {session_id}"
         );
 
@@ -708,23 +819,26 @@ impl SignallableImpl for Signaller {
     }
 
     fn end_session(&self, session_id: &str) {
-        gst::debug!(CAT, imp: self, "Signalling session done {}", session_id);
+        gst::debug!(CAT, imp = self, "Signalling session done {}", session_id);
 
         let state = self.state.lock().unwrap();
         let session_id = session_id.to_string();
         if let Some(mut sender) = state.websocket_sender.clone() {
-            RUNTIME.spawn(glib::clone!(@weak self as this => async move {
-                if let Err(err) = sender
-                    .send(p::IncomingMessage::EndSession(p::EndSessionMessage {
-                        session_id,
-                    }))
-                    .await
-                {
-                    this.obj().emit_by_name::<()>("error", &[&format!("Error: {}", err)]);
+            RUNTIME.spawn(glib::clone!(
+                #[to_owned(rename_to = this)]
+                self,
+                async move {
+                    if let Err(err) = sender
+                        .send(p::IncomingMessage::EndSession(p::EndSessionMessage {
+                            session_id,
+                        }))
+                        .await
+                    {
+                        this.obj()
+                            .emit_by_name::<()>("error", &[&format!("Error: {err}")]);
+                    }
                 }
-            }));
+            ));
         }
     }
 }
-
-impl GstObjectImpl for Signaller {}
